syntax = "proto3";

package xray.common.serial;
option csharp_namespace = "Xray.Common.Serial";
option go_package = "github.com/xtls/xray-core/common/serial";
option java_package = "com.xray.common.serial";
option java_multiple_files = true;

// TypedMessage is a serialized proto message along with its type name.
message TypedMessage {
  // The name of the message type, retrieved from protobuf API.
  string type = 1;
  // Serialized proto message.
  bytes value = 2;
}
