syntax = "proto3";

package xray.app.metrics;
option csharp_namespace = "Xray.App.Metrics";
option go_package = "github.com/xtls/xray-core/app/metrics";
option java_package = "com.xray.app.metrics";
option java_multiple_files = true;

// Config is the settings for metrics.
message Config {
  // Tag of the outbound handler that handles metrics http connections.
  string tag = 1;
  string listen = 2;
}
