syntax = "proto3";

package xray.app.version;
option csharp_namespace = "Xray.App.Version";
option go_package = "github.com/xtls/xray-core/app/version";
option java_package = "com.xray.app.version";
option java_multiple_files = true;


message Config {
  string core_version = 1;
  string min_version = 2;
  string max_version = 3;
}
