// Package kcp - A Fast and Reliable ARQ Protocol
//
// Acknowledgement:
//
//	skywind3000@github for inventing the KCP protocol
//	xtaci@github for translating to Golang
package kcp

const protocolName = "mkcp"
