syntax = "proto3";

package xray.app.dispatcher;
option csharp_namespace = "Xray.App.Dispatcher";
option go_package = "github.com/xtls/xray-core/app/dispatcher";
option java_package = "com.xray.app.dispatcher";
option java_multiple_files = true;

message SessionConfig {
  reserved 1;
}

message Config {
  SessionConfig settings = 1;
}
