syntax = "proto3";

package xray.transport.internet.tcp;
option csharp_namespace = "Xray.Transport.Internet.Tcp";
option go_package = "github.com/xtls/xray-core/transport/internet/tcp";
option java_package = "com.xray.transport.internet.tcp";
option java_multiple_files = true;

import "common/serial/typed_message.proto";

message Config {
  reserved 1;
  xray.common.serial.TypedMessage header_settings = 2;
  bool accept_proxy_protocol = 3;
}
