syntax = "proto3";

package xray.common.protocol;
option csharp_namespace = "Xray.Common.Protocol";
option go_package = "github.com/xtls/xray-core/common/protocol";
option java_package = "com.xray.common.protocol";
option java_multiple_files = true;

import "common/serial/typed_message.proto";

// User is a generic user for all protocols.
message User {
  uint32 level = 1;
  string email = 2;

  // Protocol specific account information. Must be the account proto in one of
  // the proxies.
  xray.common.serial.TypedMessage account = 3;
}
