syntax = "proto3";

package xray.proxy.blackhole;
option csharp_namespace = "Xray.Proxy.Blackhole";
option go_package = "github.com/xtls/xray-core/proxy/blackhole";
option java_package = "com.xray.proxy.blackhole";
option java_multiple_files = true;

import "common/serial/typed_message.proto";

message NoneResponse {}

message HTTPResponse {}

message Config {
  xray.common.serial.TypedMessage response = 1;
}
