syntax = "proto3";

package xray.proxy.vless;
option csharp_namespace = "Xray.Proxy.Vless";
option go_package = "github.com/xtls/xray-core/proxy/vless";
option java_package = "com.xray.proxy.vless";
option java_multiple_files = true;

message Reverse {
  string tag = 1;
}

message Account {
  // ID of the account, in the form of a UUID, e.g., "66ad4540-b58c-4ad2-9926-ea63445a9b57".
  string id = 1;
  // Flow settings. May be "xtls-rprx-vision".
  string flow = 2;

  string encryption = 3;
  uint32 xorMode = 4;
  uint32 seconds = 5;
  string padding = 6;

  Reverse reverse = 7;

  uint32 testpre = 8;
  repeated uint32 testseed = 9;
}
