syntax = "proto3";

package xray.transport.internet.headers.tls;
option csharp_namespace = "Xray.Transport.Internet.Headers.Tls";
option go_package = "github.com/xtls/xray-core/transport/internet/headers/tls";
option java_package = "com.xray.transport.internet.headers.tls";
option java_multiple_files = true;

message PacketConfig {}
